@@ -1,10 +1,12 @@
+# © Copyright Databand.ai, an IBM Company 2022
+
 import logging
 
 from contextlib import contextmanager
 from itertools import islice
 from typing import Optional
 
-from dbnd import get_dbnd_project_config, log_metric, log_metrics
+from dbnd import config, get_dbnd_project_config, log_metric, log_metrics
 from dbnd._core.constants import TaskRunState
 from dbnd._core.errors.errors_utils import log_exception
 from dbnd._core.log.dbnd_log import dbnd_log_exception, dbnd_log_tracking
@@ -28,6 +30,7 @@ logger = logging.getLogger(__name__)
 
 AIRFLOW_MONITOR_CONFIG_NAME = "airflow_monitor"
 DAG_IDS_FOR_TRACKING_CONFIG_NAME = "dag_ids"
+IS_SYNC_ENABLED_TRACKING_CONFIG_NAME = "is_sync_enabled"
 
 
 def get_tracking_dag_ids_from_airflow_json():
@@ -37,10 +40,14 @@ def get_tracking_dag_ids_from_airflow_json():
         )
 
         json = get_dbnd_json_config_from_airflow_connections()
-
         if json:
             monitor_config = json.get(AIRFLOW_MONITOR_CONFIG_NAME, None)
             if monitor_config:
+                is_sync_enabled = monitor_config.get(
+                    IS_SYNC_ENABLED_TRACKING_CONFIG_NAME, True
+                )
+                if not is_sync_enabled:
+                    return []
                 dag_ids_config = monitor_config.get(
                     DAG_IDS_FOR_TRACKING_CONFIG_NAME, None
                 )
@@ -120,9 +127,11 @@ def new_execute(context):
 
     # running the operator's original execute function
     try:
-        with af_tracking_context(task_run, context, copied_operator):
-            execute = get_execute_function(copied_operator)
-            result = execute(copied_operator, context)
+        operator_config = getattr(copied_operator, "dbnd_config", {})
+        with config(operator_config, "airflow_operator_definition"):
+            with af_tracking_context(task_run, context, copied_operator):
+                execute = get_execute_function(copied_operator)
+                result = execute(copied_operator, context)
 
     # Check if this is sensor task that is retrying - normal behavior and not really an exception
     except AirflowRescheduleException:
